@@ -8,9 +8,9 @@ in the user_context module
 from collections import namedtuple
 
 from dagster import check
-from dagster.core.definitions.executable import ExecutablePipeline
 from dagster.core.definitions.hook import HookDefinition
 from dagster.core.definitions.mode import ModeDefinition
+from dagster.core.definitions.pipeline_base import IPipeline
 from dagster.core.definitions.resource import ScopedResourcesBuilder
 from dagster.core.definitions.step_launcher import StepLauncher
 from dagster.core.errors import DagsterInvariantViolationError
@@ -28,7 +28,7 @@ class SystemExecutionContextData(
         (
             "pipeline_run scoped_resources_builder environment_config pipeline "
             "mode_def system_storage_def intermediate_storage_def instance intermediate_storage file_manager "
-            "raise_on_error retries"
+            "raise_on_error retries execution_plan"
         ),
     )
 ):
@@ -51,11 +51,13 @@ class SystemExecutionContextData(
         file_manager,
         raise_on_error,
         retries,
+        execution_plan,
     ):
         from dagster.core.definitions.system_storage import SystemStorageDefinition
         from dagster.core.definitions.intermediate_storage import IntermediateStorageDefinition
         from dagster.core.storage.intermediate_storage import IntermediateStorage
         from dagster.core.instance import DagsterInstance
+        from dagster.core.execution.plan.plan import ExecutionPlan
 
         return super(SystemExecutionContextData, cls).__new__(
             cls,
@@ -66,7 +68,7 @@ class SystemExecutionContextData(
             environment_config=check.inst_param(
                 environment_config, "environment_config", EnvironmentConfig
             ),
-            pipeline=check.inst_param(pipeline, "pipeline", ExecutablePipeline),
+            pipeline=check.inst_param(pipeline, "pipeline", IPipeline),
             mode_def=check.inst_param(mode_def, "mode_def", ModeDefinition),
             system_storage_def=check.inst_param(
                 system_storage_def, "system_storage_def", SystemStorageDefinition
@@ -81,6 +83,7 @@ class SystemExecutionContextData(
             file_manager=check.inst_param(file_manager, "file_manager", FileManager),
             raise_on_error=check.bool_param(raise_on_error, "raise_on_error"),
             retries=check.inst_param(retries, "retries", Retries),
+            execution_plan=check.inst_param(execution_plan, "execution_plan", ExecutionPlan),
         )
 
     @property
@@ -173,6 +176,10 @@ class SystemExecutionContext(object):
     def logging_tags(self):
         return self._log_manager.logging_tags
 
+    @property
+    def execution_plan(self):
+        return self._execution_context_data.execution_plan
+
     def has_tag(self, key):
         check.str_param(key, "key")
         return key in self.logging_tags
