@@ -25,6 +25,7 @@ from dagster.core.execution.resolve_versions import join_and_hash, resolve_step_
 from dagster.core.storage.migration.utils import upgrading_instance
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus, PipelineRunsFilter
 from dagster.core.storage.tags import MEMOIZED_RUN_TAG
+from dagster.core.system_config.objects import EnvironmentConfig
 from dagster.core.utils import str_format_list
 from dagster.serdes import ConfigurableClass, whitelist_for_serdes
 from dagster.seven import get_current_datetime_in_utc
@@ -47,7 +48,7 @@ def _is_dagster_home_set():
     return bool(os.getenv("DAGSTER_HOME"))
 
 
-def _is_memoized_run(tags):
+def is_memoized_run(tags):
     return tags is not None and MEMOIZED_RUN_TAG in tags and tags.get(MEMOIZED_RUN_TAG) == "true"
 
 
@@ -491,32 +492,34 @@ class DagsterInstance:
     def get_run_group(self, run_id):
         return self._run_storage.get_run_group(run_id)
 
-    def resolve_unmemoized_steps(self, execution_plan, run_config, mode):
+    def resolve_memoized_execution_plan(self, execution_plan, run_config, mode):
         """
         Returns:
-            List[str]: Step keys for all steps that don't have existing results stored for their
-                versions.
+            ExecutionPlan: Execution plan configured to only run unmemoized steps.
         """
-        pipeline_name = execution_plan.pipeline.get_definition().name
+        pipeline_def = execution_plan.pipeline.get_definition()
+        pipeline_name = pipeline_def.name
+
         step_output_versions = resolve_step_output_versions(
-            execution_plan, run_config=run_config, mode=mode
+            execution_plan,
+            EnvironmentConfig.build(pipeline_def, run_config, mode),
+            pipeline_def.get_mode_definition(mode),
         )
+        if all(version is None for version in step_output_versions.values()):
+            raise DagsterInvariantViolationError(
+                "While creating a memoized pipeline run, no steps have versions. At least one step "
+                "must have a version."
+            )
 
-        for step_output_handle, version in step_output_versions.items():
-            if version is None:
-                raise DagsterInvariantViolationError(
-                    "While creating a memoized pipeline run, a version is None for step "
-                    "{step_output}. Versions must be non-null values when running a memoized "
-                    "pipeline.".format(step_output=step_output_handle.step_key)
-                )
         step_output_addresses = self.get_addresses_for_step_output_versions(
             {
                 (pipeline_name, step_output_handle): version
                 for step_output_handle, version in step_output_versions.items()
+                if version
             }
         )
 
-        return list(
+        step_keys_to_execute = list(
             {
                 step_output_handle.step_key
                 for step_output_handle in step_output_versions.keys()
@@ -524,6 +527,8 @@ class DagsterInstance:
             }
         )
 
+        return execution_plan.build_memoized_plan(step_keys_to_execute, step_output_addresses)
+
     def create_run_for_pipeline(
         self,
         pipeline_def,
@@ -578,22 +583,23 @@ class DagsterInstance:
             len(full_execution_plan.step_keys_to_execute) == len(full_execution_plan.steps)
         )
 
-        if _is_memoized_run(tags):
+        if is_memoized_run(tags):
             if step_keys_to_execute:
                 raise DagsterInvariantViolationError(
                     "step_keys_to_execute parameter cannot be used in conjunction with memoized "
                     "pipeline runs."
                 )
 
-            step_keys_to_execute = self.resolve_unmemoized_steps(
+            subsetted_execution_plan = self.resolve_memoized_execution_plan(
                 full_execution_plan, run_config=run_config, mode=mode,
             )  # TODO: tighter integration with existing step_keys_to_execute functionality
-
-        subsetted_execution_plan = (
-            full_execution_plan.build_subset_plan(step_keys_to_execute)
-            if step_keys_to_execute
-            else full_execution_plan
-        )
+            step_keys_to_execute = subsetted_execution_plan.step_keys_to_execute
+        else:
+            subsetted_execution_plan = (
+                full_execution_plan.build_subset_plan(step_keys_to_execute)
+                if step_keys_to_execute
+                else full_execution_plan
+            )
 
         return self.create_run(
             pipeline_name=pipeline_def.name,
