@@ -14,7 +14,7 @@ class TempfileManager(object):
 
     def tempfile(self):
 
-        temporary_file = tempfile.NamedTemporaryFile('w+b', delete=False)
+        temporary_file = tempfile.NamedTemporaryFile("w+b", delete=False)
         self.files.append(temporary_file)
         self.paths.append(temporary_file.name)
         return temporary_file
